#
# script to generate vertex and fragment shaders for Gaussian blur with given kernel size
#

import sys, math, string

						# 1d Gaussian distribution, s is standard deviation
def gaussian ( x, s ):
	return math.exp ( -x*x/(2*s*s) )

						# return array normalized of coefficients for kernal size k
def buildCoeffs ( k, s ):
	coeffs = []	
	sum    = -1.0
	for x in range ( k + 1 ):
		c    = gaussian ( float ( x ), s )
		sum += 2*c
		coeffs.append ( c )
	
						# renormalize coeffs
	for x in range ( k + 1 ):
		coeffs [x] = coeffs [x] / sum
	
	return coeffs

def genVertexShader ( k, s ):
	return "void main(void)\n{\n\tgl_Position     = gl_ModelViewProjectionMatrix * gl_Vertex;\n\tgl_TexCoord [0] = gl_MultiTexCoord0;\n}\n"

def genFragmentShader ( k, s, d1, d2 ):
	coeffs = buildCoeffs ( k, s )
	shd1   = "uniform	sampler2D	mainTex;\nvoid main (void)\n{\n\tvec2 tx  = gl_TexCoord [0].xy;\n"
	shd2   = "\tvec2 dx  = vec2 (%f,%f);\n" % ( d1, d2 )
	shd3   = "\tvec2 sdx = dx;\n"
	shd4   = "\tvec4 sum = texture2D ( mainTex, tx ) * %f;\n\n" % coeffs [0]
	shd    = shd1 + shd2 + shd3 + shd4
	
	for x in range ( 1, k + 1 ):
		shd = shd + ( "\tsum += (texture2D ( mainTex, tx + sdx ) + texture2D ( mainTex, tx - sdx ) )* %f; \n" % coeffs [x] )
		shd = shd + "\tsdx += dx;\n"
		
	return shd + "\n\tgl_FragColor = sum;\n}\n"


def getOpts ():
	opt  = ""
	opts = {}
	for s in sys.argv[1:]:
		if s [0] == '-':
			opt = s[1:]
		else:
			opts [opt] = s
			opt        = ""
	
	return opts

			
if __name__ == "__main__":
	s    = 3.0
	k    = 7
	name = "b-y"
	
	if len ( sys.argv ) > 1:					# there're some arguments
		if sys.argv [1] == '-h' or sys.argv [1] == '-?':
			print "Gaussian blure shader generator"
			print "Options:"
			print "     -s n        - set the S constant"
			print "     -k n        - set the kernel radius"
			print "     -f filename - set the output filename"
			sys.exit ( 0 )
			
		opts = getOpts ()
		
		if opts.has_key ( "s" ):
			s = string.atof ( opts ["s"] )
			
		if opts.has_key ( "k" ):
			k = string.atoi ( opts ["k"] )
		
		if opts.has_key ( "f" ):
			name = opts ["f"]
		
	vert = genVertexShader ( k, s )
	frag = genFragmentShader ( k, s, 0.0, 1.0 / 512.0 )
	f1   = open ( name + ".vsh", "w" )
	f1.write ( vert )
	f1.close ()
	f2   = open ( name + ".fsh", "w" )
	f2.write ( frag )
	f2.close ()
	print "Shaders %s and %s have been successfully generated" % ( name + ".vsh", name + ".fsh" )
