//
// Simple test for texture loaders
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include	"libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include	<glut.h>
#endif

#include	<stdio.h>
#include	<stdlib.h>

#ifdef	_WIN32
    #include 	<gl\glaux.h>
#endif

#include	"libTexture.h"
#include	"rgbe.h"
#include	"GlslProgram.h"

GlslProgram program;
unsigned	decalMap;

/////////////////////////////////////////////////////////////////////////////////

void init ()
{
	glClearColor ( 0.0, 0.0, 0.0, 1.0 );
	glEnable     ( GL_DEPTH_TEST );
	glDepthFunc  ( GL_LEQUAL );

	glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
	glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void    startOrtho ()
{
    glMatrixMode   ( GL_PROJECTION );                   // select the projection matrix
    glPushMatrix   ();                                  // store the projection matrix
    glLoadIdentity ();                                  // reset the projection matrix
                                                        // set up an ortho screen
    glOrtho        ( 0, 512, 0, 512, -1, 1 );
    glMatrixMode   ( GL_MODELVIEW );                    // select the modelview matrix
    glPushMatrix   ();                                  // store the modelview matrix
    glLoadIdentity ();                                  // reset the modelview matrix
}

void    endOrtho ()
{
    glMatrixMode ( GL_PROJECTION );                     // select the projection matrix
    glPopMatrix  ();                                    // restore the old projection matrix
    glMatrixMode ( GL_MODELVIEW );                      // select the modelview matrix
    glPopMatrix  ();                                    // restore the old projection matrix
}

void display ()
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	startOrtho ();
	glEnable      ( GL_TEXTURE_RECTANGLE_ARB );
	glBindTexture ( GL_TEXTURE_RECTANGLE_ARB, decalMap );

	program.bind ();
	glBegin ( GL_QUADS );

	glTexCoord2f ( 0, 0 );
	glVertex2f   ( 0, 0 );

	glTexCoord2f ( 0, 500 );
	glVertex2f   ( 0, 511 );

	glTexCoord2f ( 500,   500  );
	glVertex2f   ( 511, 511 );

	glTexCoord2f ( 500, 0 );
	glVertex2f   ( 511, 0 );

	glEnd ();

	program.unbind ();
	endOrtho ();

	glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
}

void key ( unsigned char key, int x, int y )
{
	if ( key == 27 || key == 'q' || key == 'Q' )	//	quit requested
    	exit ( 0 );
}

unsigned loadTex ( const char * fileName )
{
	Data * data = getFile ( fileName );
	
	if ( data == NULL )
		return 0;
		
	decalMap = loadRgbeAsRgba ( data, GL_TEXTURE_RECTANGLE_ARB, GL_RGBA );
	
	delete data;
}

int main ( int argc, char * argv [] )
{
								// initialize glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize  ( 512, 512 );


								// create window
	glutCreateWindow ( "Simple test for image loaders" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
	glutKeyboardFunc ( key     );

	init           ();
	initExtensions ();

    if ( !program.loadShaders ( "tex-hdr.vsh", "tex-hdr.fsh" ) )
    {
        printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

        return 3;
    }

	loadTex ( "images/nave.hdr" );

    program.bind            ();
    program.setTexture      ( "mainTex", 0 );
    program.unbind          ();
	
    glutMainLoop ();

	return 0;
}
